@@ -26,6 +26,11 @@
 
 ;;; Code:
 
+;; Optional dependencies.
+(and (require 'targets/elpa nil t)
+     (fboundp 'ivy--elpa-activate)
+     (ivy--elpa-activate))
+
 (defvar ivy-empty "tests/find-file/empty-dir/")
 
 (defvar ivy-features nil
@@ -49,8 +54,6 @@ Intended as :after-while advice for `require'."
 
 (require 'ert)
 
-(message "%s" (emacs-version))
-
 (setq ivy-last (make-ivy-state))
 
 (ert-deftest ivy--lazy-load-ffap--ffap-url-p ()
@@ -576,10 +579,10 @@ Since `execute-kbd-macro' doesn't pick up a let-bound `default-directory'.")
              (colir-blend-face-background 0 (length str) 'ivy-current-match str)
              str)
            #("Desktop" 0 7 (face (ivy-current-match (foreground-color . "#8ac6f2") bold))))))
+
+;;; Prefix arg tests
+;;;; Tests with no prefix
 
-
-;;* prefix arg tests
-;;** tests with no prefix
 (ert-deftest ivy-no-prefix-arg ()
   "Tests with no prefix arg."
   (should (equal
@@ -655,7 +658,8 @@ Since `execute-kbd-macro' doesn't pick up a let-bound `default-directory'.")
             "TAB TAB")
            nil)))
 
-;;** tests with one prefix
+;;;; Tests with one prefix
+
 (ert-deftest ivy-one-prefix-arg ()
   "Tests with no prefix arg."
   (should (equal
@@ -1010,7 +1014,8 @@ Since `execute-kbd-macro' doesn't pick up a let-bound `default-directory'.")
   (mapc (lambda (pair)
           (dolist (str (cdr pair))
             (ert-info ((format "%S" str) :prefix "String: ")
-              (should (= (ivy-completion-common-length str) (car pair))))))
+              (should (= (with-no-warnings (ivy-completion-common-length str))
+                         (car pair))))))
         '((0 ""
              #("a"   0 1 (face completions-first-difference))
              #("ab"  0 1 (face completions-first-difference))
