@@ -1,12 +1,12 @@
 ;;; swiper.el --- Isearch with an overview.  Oh, man! -*- lexical-binding: t -*-
 
-;; Copyright (C) 2015-2023 Free Software Foundation, Inc.
+;; Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
 ;; Author: Oleh Krehel <ohwoeowho@gmail.com>
-;; Maintainer: Basil L. Contovounesios <contovob@tcd.ie>
+;; Maintainer: Basil L. Contovounesios <basil@contovou.net>
 ;; URL: https://github.com/abo-abo/swiper
-;; Version: 0.14.2
-;; Package-Requires: ((emacs "24.5") (ivy "0.14.2"))
+;; Version: 0.15.1
+;; Package-Requires: ((emacs "24.5") (ivy "0.15.1"))
 ;; Keywords: matching
 
 ;; This file is part of GNU Emacs.
@@ -38,6 +38,16 @@
 (require 'cl-lib)
 (require 'ivy)
 
+(eval-when-compile
+  (unless (fboundp 'static-if)
+    (defmacro static-if (condition then-form &rest else-forms)
+      "Expand to THEN-FORM or ELSE-FORMS based on compile-time CONDITION.
+Polyfill for Emacs 30 `static-if'."
+      (declare (debug (sexp sexp &rest sexp)) (indent 2))
+      (if (eval condition lexical-binding)
+          then-form
+        (macroexp-progn else-forms)))))
+
 (defgroup swiper nil
   "`isearch' with an overview."
   :group 'matching
@@ -145,12 +155,12 @@ If the input is empty, select the previous history element instead."
 
 (defvar swiper-map
   (let ((map (make-sparse-keymap)))
-    (define-key map (kbd "C-s") 'swiper-C-s)
-    (define-key map (kbd "M-q") 'swiper-query-replace)
-    (define-key map (kbd "C-l") 'swiper-recenter-top-bottom)
-    (define-key map (kbd "C-'") 'swiper-avy)
-    (define-key map (kbd "C-7") 'swiper-mc)
-    (define-key map (kbd "C-c C-f") 'swiper-toggle-face-matching)
+    (define-key map (kbd "C-s") #'swiper-C-s)
+    (define-key map (kbd "M-q") #'swiper-query-replace)
+    (define-key map (kbd "C-l") #'swiper-recenter-top-bottom)
+    (define-key map (kbd "C-'") #'swiper-avy)
+    (define-key map (kbd "C-7") #'swiper-mc)
+    (define-key map (kbd "C-c C-f") #'swiper-toggle-face-matching)
     map)
   "Keymap for swiper.")
 
@@ -237,7 +247,7 @@ If the input is empty, select the previous history element instead."
 
 (ivy-configure 'swiper-query-replace
   :update-fn #'swiper--query-replace-updatefn)
-(put 'swiper-query-replace 'no-counsel-M-x t)
+(function-put #'swiper-query-replace 'no-counsel-M-x t)
 
 (defvar inhibit-message)
 
@@ -262,7 +272,7 @@ If the input is empty, select the previous history element instead."
                     (goto-char (point-min))
                     (perform-replace from to t t nil)))
              (set-window-configuration wnd-conf))))))))
-(put 'swiper-all-query-replace 'no-counsel-M-x t)
+(function-put #'swiper-all-query-replace 'no-counsel-M-x t)
 
 (defvar avy-all-windows)
 (defvar avy-style)
@@ -280,18 +290,19 @@ If the input is empty, select the previous history element instead."
 (declare-function avy--remove-leading-chars "ext:avy")
 
 (defun swiper--avy-candidates ()
-  (let* (
+  (let* ((visible-overlays
+          (with-ivy-window (overlays-in (window-start) (window-end))))
          ;; We'll have overlapping overlays, so we sort all the
          ;; overlays in the visible region by their start, and then
          ;; throw out non-Swiper overlays or overlapping Swiper
          ;; overlays.
-         (visible-overlays (cl-sort (with-ivy-window
-                                      (overlays-in (window-start)
-                                                   (window-end)))
-                                    #'< :key #'overlay-start))
+         (visible-overlays
+          (static-if (bound-and-true-p ivy--new-sort-p)
+              (sort visible-overlays :key #'overlay-start :in-place t)
+            (cl-sort visible-overlays #'< :key #'overlay-start)))
          (min-overlay-start 0)
          (overlays-for-avy
-          (cl-remove-if-not
+          (cl-delete-if-not
            (lambda (ov)
              (when (and (>= (overlay-start ov)
                             min-overlay-start)
@@ -470,7 +481,7 @@ such as `scroll-conservatively' are set to a high value.")
   "List of major-modes that are incompatible with `font-lock-ensure'.")
 
 (defun swiper-font-lock-ensure-p ()
-  "Return non-nil if we should `font-lock-ensure'."
+  "Return non-nil if we should not `font-lock-ensure'."
   (or (derived-mode-p 'magit-mode)
       (bound-and-true-p magit-blame-mode)
       (memq major-mode swiper-font-lock-exclude)
@@ -478,12 +489,13 @@ such as `scroll-conservatively' are set to a high value.")
 
 (defun swiper-font-lock-ensure ()
   "Ensure the entire buffer is highlighted."
-  (unless (swiper-font-lock-ensure-p)
-    (unless (or (> (buffer-size) 100000) (null font-lock-mode))
-      (if (fboundp 'font-lock-ensure)
-          ;; Added in Emacs 25.1.
-          (font-lock-ensure)
-        (with-no-warnings (font-lock-fontify-buffer))))))
+  (unless (or (swiper-font-lock-ensure-p)
+              (> (buffer-size) 100000)
+              (not font-lock-mode))
+    (static-if (fboundp 'font-lock-ensure)
+        ;; Added in Emacs 25.1.
+        (font-lock-ensure)
+      (font-lock-fontify-buffer))))
 
 (defvar swiper--format-spec ""
   "Store the current candidates format spec.")
@@ -770,7 +782,7 @@ Add a space after a leading `^' for `swiper', and apply
 This is the regex used in the minibuffer where candidates have
 line numbers.  For the buffer, use `ivy--regex' instead."
   (let* ((re-builder (ivy-alist-setting ivy-re-builders-alist))
-         (str (replace-regexp-in-string "\\\\n" "\n" str))
+         (str (ivy--string-replace "\\n" "\n" str))
          (re (funcall re-builder str)))
     (if (consp re)
         (mapcar
@@ -846,6 +858,9 @@ When non-nil, INITIAL-INPUT is the initial search pattern."
   :unwind-fn #'swiper--cleanup
   :index-fn #'ivy-recompute-index-swiper)
 
+(ivy-add-actions 'swiper
+                 `(("w" ,#'swiper-action-copy "copy")))
+
 (defun swiper-toggle-face-matching ()
   "Toggle matching only the candidates with `swiper-invocation-face'."
   (interactive)
@@ -937,24 +952,17 @@ the face, window and priority of the overlay."
   (or (display-graphic-p)
       (not recenter-redisplay)))
 
-(defun swiper--positive-regexps ()
-  (if (listp ivy-regex)
-      (mapcar #'car (cl-remove-if-not #'cdr ivy-regex))
-    (list ivy-regex)))
-
 (defun swiper--update-input-ivy ()
   "Called when `ivy' input is updated."
   (with-ivy-window
     (swiper--cleanup)
     (when (> (length (ivy-state-current ivy-last)) 0)
-      (let ((regexps (swiper--positive-regexps))
+      (let ((regexps (ivy--positive-regexps))
             (re-idx -1)
             (case-fold-search (ivy--case-fold-p ivy-text)))
         (dolist (re regexps)
           (setq re-idx (1+ re-idx))
-          (let* ((re (replace-regexp-in-string
-                      "    " "\t"
-                      re))
+          (let* ((re (ivy--string-replace "    " "\t" re))
                  (num (swiper--line-number (ivy-state-current ivy-last))))
             (unless (memq this-command '(ivy-yank-word
                                          ivy-yank-symbol
@@ -1081,7 +1089,7 @@ WND, when specified is the window."
 (defvar evil-ex-search-pattern)
 (defvar evil-ex-search-persistent-highlight)
 (defvar evil-ex-search-direction)
-(declare-function evil-ex-search-activate-highlight "evil-ex")
+(declare-function evil-ex-search-activate-highlight "ext:evil-search")
 
 (defun swiper--maybe-recenter ()
   (cond (swiper-action-recenter
@@ -1255,13 +1263,14 @@ otherwise continue prompting for buffers."
       ((eq mode 'gnus-article-mode) t)
       ;; Otherwise, only consider the file if it's backed by a file.
       (t (buffer-file-name buffer)))))
+
+;;; `swiper-all'
 
-;;* `swiper-all'
 (defun swiper-all-function (str)
   "Search in all open buffers for STR."
   (or
    (ivy-more-chars)
-   (let* ((buffers (cl-remove-if-not #'swiper-all-buffer-p (buffer-list)))
+   (let* ((buffers (cl-delete-if-not #'swiper-all-buffer-p (buffer-list)))
           (re-full ivy-regex)
           re re-tail
           cands match
@@ -1327,7 +1336,7 @@ See `ivy-format-functions-alist' for further information."
 
 (defvar swiper-all-map
   (let ((map (make-sparse-keymap)))
-    (define-key map (kbd "M-q") 'swiper-all-query-replace)
+    (define-key map (kbd "M-q") #'swiper-all-query-replace)
     map)
   "Keymap for `swiper-all'.")
 
@@ -1375,8 +1384,9 @@ See `ivy-format-functions-alist' for further information."
                 (swiper--candidates 4))
                res))))
     res))
+
+;;; `swiper-isearch'
 
-;;* `swiper-isearch'
 (defun swiper-isearch-function (str)
   "Collect STR matches in the current buffer for `swiper-isearch'."
   (with-ivy-window
@@ -1483,7 +1493,7 @@ that we search only for one character."
              (lambda ()
                (with-ivy-window
                  (swiper--add-overlays (ivy--regex ivy-text))))))
-    (dolist (re (swiper--positive-regexps))
+    (dolist (re (ivy--positive-regexps))
       (swiper--add-overlays re))))
 
 (defun swiper--isearch-candidate-pos (cand)
@@ -1560,10 +1570,6 @@ completion."
     ;; In case of unexpected error.
     (goto-char swiper--opoint)))
 
-(ivy-add-actions 'swiper-isearch '(("w" swiper-isearch-action-copy "copy")))
-(ivy-add-actions 'swiper-isearch '(("i" swiper-isearch-action-insert "insert")))
-(ivy-add-actions 'swiper '(("w" swiper-action-copy "copy")))
-
 (defun swiper--isearch-insert-current ()
   "Replace minibuffer contents with the current candidate.
 Like `ivy-insert-current', but tailored for `swiper-isearch'."
@@ -1572,6 +1578,16 @@ Like `ivy-insert-current', but tailored for `swiper-isearch'."
   (let ((cur (ivy-state-current ivy-last)))
     (insert (with-ivy-window (swiper--isearch-candidate-string cur)))))
 
+(defun swiper--isearch-kill-ring-save ()
+  "Save the current candidates in the kill ring.
+If the region is active, forward to `kill-ring-save' instead.
+Like `ivy-kill-ring-save', but tailored for `swiper-isearch'."
+  (interactive)
+  (if (use-region-p)
+      (call-interactively #'kill-ring-save)
+    (kill-new (with-ivy-window
+                (mapconcat #'swiper--line-at-point ivy--old-cands "\n")))))
+
 (defun swiper-isearch-thing-at-point ()
   "Insert `symbol-at-point' into the minibuffer of `swiper-isearch'.
 When not running `swiper-isearch' already, start it."
@@ -1612,7 +1628,12 @@ When the input is empty, browse the search history instead."
 (defvar swiper-isearch-map
   (let ((map (make-sparse-keymap)))
     (set-keymap-parent map swiper-map)
-    (define-key map [remap ivy-insert-current] #'swiper--isearch-insert-current)
+    (define-key map `[remap ,#'ivy-insert-current]
+                #'swiper--isearch-insert-current)
+    (define-key map `[remap ,#'ivy-kill-ring-save]
+                #'swiper--isearch-kill-ring-save)
+    (define-key map `[remap ,#'kill-ring-save]
+                #'swiper--isearch-kill-ring-save)
     (define-key map (kbd "M-n") #'swiper-isearch-thing-at-point)
     (define-key map (kbd "C-r") #'swiper-isearch-C-r)
     map)
@@ -1761,6 +1782,10 @@ When the input is empty, browse the search history instead."
   :unwind-fn #'swiper--isearch-unwind
   :format-fn #'swiper-isearch-format-function)
 
+(ivy-add-actions 'swiper-isearch
+                 `(("w" ,#'swiper-isearch-action-copy "copy")
+                   ("i" ,#'swiper-isearch-action-insert "insert")))
+
 ;;;###autoload
 (defun swiper-isearch-backward (&optional initial-input)
   "Like `swiper-isearch' but the first result is before the point."
